/***********************************************************************/
/*  TSA5511 Funktionen						       */	
/*  fuer i2c-Interface 	 				               */
/*  V0.1  erstellt am  : 07.05.2001 / I. Gerlach                       */
/*  Dateiname          : tsa5511.c				       */
/*                		       				       */
/*  Aenderungen : 						       */
/*                                                                     */
/* 04.08.01 , Erweiterung tsa_struc, QuarzFrequenz                     */
/* 07.05.01 , Start 70cm Messender f. Tuner 			       */
/*                                                                     */
/***********************************************************************/

/*

Der TSA5511 arbeitet zwischen 64-1300MHz, Schrittweite ist 62,5 KHz. (Quarz 4,00MHz)
Der feste Vorteiler P ist 8. 
Der Variable Vorteiler N hat einen Bereich von 1 ??? -32767.
Als Frequenzteiler liegt der kleinste Bereich bei nmin = P * Nmin = 8 * 256 = 2048 

Ansteuerung TSA5511
 
		128	64	32	16	8	4	2	1	
--------------------------------------------------------------------------
Adresse:	1	1	0	0	0	A1	A0	R/W  : 192	
Teiler1:	0	N14	N13	N12	N11	N10	N9	N8 
Teiler2:	N7	N6	N5	N4	N3	N2	N1	N0
Control1:	1	CP	T1	T0	1	1	1	0S (0)   
Control2:	P7	P6	P5	P4	P3	P2	P1	P0

Betrieb:	R/W=0 		write / 1 = read
		CP=1		Charge Pump high 
		T0=T1=T2=0 	PLL
		T0=T1=T2=1	Test


*/

#include <stdlib.h>
#include <stdio.h>
#include "../src/i2c_fncs.h"
#include "tsa5511.h"


int tsa5511_init(tsa_struc *sp,int Mode,long Quarz)	// Mode 206 PLL, 238 Teiler, Rckgabe Mode
{							// Quarz wenn 0 ,dann def. 4 MHz
  memset(sp,0,sizeof(tsa_struc));		// Strucktur auf 0 setzen
  sp->cnt1=Mode;				// Mode PLL  / Teiler Charge Pump 0!!!
  sp->cnt2=2;	
  sp->quarz=TSA5511_Q;
  sp->step=TSA5511_Q/64; 
  if(Quarz!=0) {
   sp->quarz=Quarz;
   sp->step=Quarz/64;
  }
 return(Mode);
}

int tsa5511_calc(tsa_struc *sp,long Freq,int dstep) // Berechnet die Teilerfaktoren in *sp , Rckgabe Teiler 
{						    // Ist dstep = 1 , wird doppelt Schrittweite verwendet
 int n;
 int temp; // 
  // n =  (Freq*TSA5511_QDF) / (TSA5511_P * TSA5511_Q); // Berechnet wird ein ganzzahliger Teiler.
  // n =  (Freq*TSA5511_QDF) / (TSA5511_P * sp->quarz); // Berechnet wird ein ganzzahliger Teiler.
  n = Freq / (sp->quarz / (TSA5511_QDF /TSA5511_P)); 
  // printf("n : %d , TSA5511_P %d TSA5511_QDF %d Quarz %ld \n",n,TSA5511_P,TSA5511_QDF,sp->quarz);
  if(dstep==1) {
   temp = n % 2;
   if (temp != 0) n = n -1;
    //  printf("temp %d ,  n %d \n",temp,n);
  }
  temp = n & 32512;			// Oberes Nibble 
  sp->div2  = n - (n & 32512);		// Oberes Nibble holen n0-n7
  sp->div1  = temp >> 8;		// n14-n8
  //sp->cnt2=TSA5523_BS0; 		// Vorgabe BS0
  //if(Freq>TSA5523_BS1B) sp->cnt2=TSA5523_BS1;
  //if(Freq>TSA5523_BS2B) sp->cnt2=TSA5523_BS2;
  // printf("Freq %ld, Step %ld Quarz %d \n",Freq,sp->step,sp->quarz);
  return(n);				// Rckgabe : Teiler ,die tats. Freq. errechnet sich aus 
}					// Fres = ((n*TSA5523_STEP) 

int tsa5511_send(tsa_struc *sp,int Adr)   // Sendet die Daten an die PLL
{
 int ret;
 iic_start();
 ret = iic_send_byte(TSA5511_ADR+Adr);
 ret = iic_send_byte(sp->div1);
 ret = iic_send_byte(sp->div2);
 ret = iic_send_byte(sp->cnt1); 
 ret = iic_send_byte(sp->cnt2);
 iic_stop();
 return(ret);
}

int tsa5511_read(int Adr)   // Liest Daten der PLL , 
{
 int ret=1;
 iic_start();
 ret = iic_send_byte(TSA5511_ADR+Adr+1);
 if (ret) ret = iic_read_byte(0);
 iic_stop();
 return(ret);
}


