/***********************************************************************/
/*  TSA5523 Funktionen						       */	
/*  fuer i2c-Interface 	 				               */
/*  V0.1  erstellt am  : 19.02.2001 / I. Gerlach                       */
/*  Dateiname          : tsa5523.c				       */
/*                		       				       */
/*  Aenderungen : 						       */
/*                                                                     */
/* 19.02.01 , Start Test mit TEMIC Tuner 3X7291 (WinTV go)	       */
/*                                                                     */
/***********************************************************************/

/*

Der TSA5523 arbeitet zwischen 64-1400MHz, Schrittweite ist 31,25/ 50,0 / 62,5 KHz. 
Der feste Vorteiler P ist 8. 
Der Variable Vorteiler N hat einen Bereich von 1 ??? -32767.
Als Frequenzteiler liegt der kleinste Bereich bei nmin = P * Nmin = 8 * 256 = 2048 
Die Quarzfrequenz liegt bei 4.00 MHz, der Teiler Q= 1024 / 640  / 512.

Ansteuereung TSA5523
 
		128	64	32	16	8	4	2	1	
--------------------------------------------------------------------------
Adresse:	1	1	0	0	0	A1	A0	R/W  : 192	
Teiler1:	0	N14	N13	N12	N11	N10	N9	N8 
Teiler2:	N7	N6	N5	N4	N3	N2	N1	N0
Control1:	1	CP	T2	T1	T0	RSA	RSB	0   
Control2:	P7	P6	P5	P4	P3	P2	P1	P0

Betrieb:	R/W=0 		write / 1 = read
		CP=1		Charge Pump high 
		T0=1;T1=T2=0 	PLL
		T0=T1=T2=1	Test

Ref Teiler

RSA	RSB	div.	Step	fref
1	1	512	62.5	7.8125
x	0	640	50.0	6.25
0	1	1024	31.25	3.90625		

Laut Datenblatt ergeben sich folgende Bereiche (Wird ueber P4-P5-P7, geschaltet).

Bereiche:	Low  VHF	 48-168	MHz
		High VHF	175-447	MHz
		UHF		455-855	MHz

*/

#include <stdlib.h>
#include <stdio.h>
#include "../src/i2c_fncs.h"
#include "tsa5523.h"


int tsa5523_init(tsa5523 *sp,int Mode)	// Mode 206 PLL, 238 Teiler, Rckgabe Mode
{
  memset(sp,0,sizeof(tsa5523));		// Strucktur auf 0 setzen
  sp->cnt1=Mode;			// Mode PLL  / Teiler Charge Pump 0!!!
  sp->cnt2=2;	
 return(Mode);
}

int tsa5523_calc(tsa5523 *sp,long Freq,int dstep)// Berechnet die Teilerfaktoren in *sp , Rckgabe Teiler 
{						 // Ist dstep = 1 , wird doppelt Schrittweite verwendet
 int n;
 int temp; // 
  n =  ((Freq+TSA5523_ZF)*TSA5523_QDF) / (TSA5523_P * TSA5523_Q); // Berechnet wird ein ganzzahliger Teiler.
  // n = Freq / (sp->quarz / (TSA5511_QDF /TSA5511_P)); 
  if(dstep==1) {
   temp = n % 2;
   if (temp != 0) n = n -1;
   // printf("temp %d ,  n %d \n",temp,n);
  }
  temp = n & 32512;			// Oberes Nibble 
  sp->div2  = n - (n & 32512);		// Oberes Nibble holen n0-n7
  sp->div1  = temp >> 8;		// n14-n8
  sp->cnt2=TSA5523_BS0; 		// Vorgabe BS0
  if(Freq>TSA5523_BS1B) sp->cnt2=TSA5523_BS1;
  if(Freq>TSA5523_BS2B) sp->cnt2=TSA5523_BS2;
  return(n);				// Rckgabe : Teiler ,die tats. Freq. errechnet sich aus 
}					// Fres = ((n*TSA5523_STEP) - TSA5523_ZF)

int tsa5523_send(tsa5523 *sp,int Adr)   // Sendet die Daten an die PLL
{
 int ret;
 iic_start();
 ret = iic_send_byte(TSA5523_ADR+Adr);
 ret = iic_send_byte(sp->div1);
 ret = iic_send_byte(sp->div2);
 ret = iic_send_byte(sp->cnt1); 
 ret = iic_send_byte(sp->cnt2);
 iic_stop();
 return(ret);
}

int tsa5523_read(int Adr)   // Liest Daten der PLL , 
{
 int ret=1;
 iic_start();
 ret = iic_send_byte(TSA5523_ADR+Adr+1);
 if (ret) ret = iic_read_byte(0);
 iic_stop();
 return(ret);
}


